package glassfish.test;

import glassfish.Child;
import glassfish.Parent;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import junit.framework.TestCase;

public class TestIdentityMap extends TestCase
{
    private static boolean setup = false;

    protected void setUp() throws Exception
    {
        if (!setup)
        {
            Class.forName("org.hsqldb.jdbcDriver");
            Connection connection = DriverManager.getConnection("jdbc:hsqldb:mem:test", "sa", "");
            Statement statement = connection.createStatement();
            statement.execute("CREATE TABLE parent( id INTEGER )");
            statement.execute("CREATE TABLE child( id INTEGER , parent_id INTEGER)");
            statement.execute("INSERT INTO parent VALUES(1)");
            statement.execute("INSERT INTO parent VALUES(2)");
            statement.execute("INSERT INTO parent VALUES(3)");
            statement.execute("INSERT INTO child VALUES(11,1)");
            statement.execute("INSERT INTO child VALUES(12,1)");
            statement.execute("INSERT INTO child VALUES(13,1)");
            statement.execute("INSERT INTO child VALUES(21,2)");
            statement.execute("INSERT INTO child VALUES(22,2)");
            statement.execute("INSERT INTO child VALUES(31,3)");
            setup = true;
        }
    }

    public void testIdentityInsideTransaction() throws Exception
    {
        EntityManager manager = getEntityManager();
        manager.getTransaction().begin();
        checkIdentity(manager);
    }

    public void testIdentityOutsideTransaction() throws Exception
    {
        EntityManager manager = getEntityManager();
        checkIdentity(manager);
    }

    @SuppressWarnings("unchecked")
    private void checkIdentity(EntityManager manager)
    {
        Query query = manager.createQuery("SELECT e FROM Child e");
        List<Child> results = query.getResultList();
        for (Child child : results)
        {
            Parent parent = child.getParent();
            Collection<Child> children = parent.getChildren();
            assertTrue(children.contains(child));
        }
    }

    private EntityManager getEntityManager()
    {
        EntityManagerFactory factory = Persistence.createEntityManagerFactory("core");
        EntityManager manager = factory.createEntityManager();
        return manager;
    }
}
